@@ -9,23 +9,33 @@ import struct
 import sys
 from typing import Any, NoReturn, Optional, Tuple, Union, cast
 
-from ._tls import DTLSConfiguration, DTLSVersion
 from ._tls import HandshakeStep as HandshakeStep
 from ._tls import HelloVerifyRequest
 from ._tls import MbedTLSBuffer as TLSWrappedBuffer
 from ._tls import (
-    NextProtocol,
     Purpose,
     RaggedEOF,
-    TLSConfiguration,
     TLSSession,
-    TLSVersion,
     TrustStore,
     WantReadError,
     WantWriteError,
     _BaseContext,
+    _tls_from_version,
+    _tls_to_version,
     ciphers_available,
 )
+from ._tlsi import DTLSConfiguration as DTLSConfiguration
+from ._tlsi import DTLSVersion as DTLSVersion
+from ._tlsi import NextProtocol as NextProtocol
+from ._tlsi import PrivateKey as PrivateKey
+from ._tlsi import ServerNameCallback as ServerNameCallback
+from ._tlsi import TLSConfiguration as TLSConfiguration
+from ._tlsi import TLSVersion as TLSVersion
+
+if sys.version_info < (3, 8):
+    from typing_extensions import Final
+else:
+    from typing import Final
 
 if sys.version_info < (3, 10):
     from typing_extensions import TypeAlias
@@ -39,9 +49,11 @@ __all__ = (
     "DTLSVersion",
     "HelloVerifyRequest",
     "NextProtocol",
+    "PrivateKey",
     "Purpose",
     "RaggedEOF",
     "ServerContext",
+    "ServerNameCallback",
     "TLSConfiguration",
     "TLSRecordHeader",
     "TLSSession",
@@ -70,10 +82,24 @@ class TLSRecordHeader:
         HANDSHAKE = 0x16
         APPLICATION_DATA = 0x17
 
-    def __init__(self, record_type: int, version: int, length: int) -> None:
-        self.record_type = record_type
-        self.version = version
-        self.length = length
+    def __init__(
+        self,
+        record_type: Union[int, TLSRecordHeader.RecordType],
+        version: Union[int, Tuple[int, int], TLSVersion],
+        length: int,
+    ) -> None:
+        def parse_version(
+            v: Union[int, Tuple[int, int], TLSVersion]
+        ) -> TLSVersion:
+            if isinstance(v, TLSVersion):
+                return v
+            if isinstance(v, int):
+                return TLSVersion(v)
+            return TLSVersion(((v[0] & 0xFF) << 8) + v[1] & 0xFF)
+
+        self.record_type: Final = TLSRecordHeader.RecordType(record_type)
+        self.version: Final = parse_version(version)
+        self.length: Final = length
 
     def __str__(self) -> str:
         return "%s(%s, %s, %s)" % (
@@ -101,24 +127,30 @@ class TLSRecordHeader:
         )
 
     def __hash__(self) -> int:
-        return 0x5AFE ^ self.record_type ^ self.version ^ self.length
+        return 0x5AFE ^ self.record_type ^ self.version.value ^ self.length
 
     def __len__(self) -> int:
         return 5
 
     def __bytes__(self) -> bytes:
+        maj, min = _tls_from_version(self.version)
+        version = ((maj & 0xFF) << 8) + (min & 0xFF)
         return struct.pack(
-            TLSRecordHeader.fmt, self.record_type, self.version, self.length
+            TLSRecordHeader.fmt,
+            self.record_type,
+            version,
+            self.length,
         )
 
     @classmethod
     def from_bytes(cls, header: bytes) -> TLSRecordHeader:
-        record_type, version, length = struct.unpack(
+        record_type, maj_min_version, length = struct.unpack(
             TLSRecordHeader.fmt, header[:5]
         )
+        maj, min = (maj_min_version >> 8) & 0xFF, maj_min_version & 0xFF
         return cls(
             TLSRecordHeader.RecordType(record_type),
-            TLSVersion(version),
+            _tls_to_version((maj, min)),
             length,
         )
 
@@ -183,6 +215,22 @@ class TLSWrappedSocket:
         self._context = buffer.context
         self._closed = False
 
+    @property
+    def context(self) -> _BaseContext:
+        return self._buffer.context
+
+    @property
+    def _buffer(self) -> TLSWrappedBuffer:
+        return cast(TLSWrappedBuffer, self.__dict__["_buffer"])
+
+    @_buffer.setter
+    def _buffer(self, __buffer: TLSWrappedBuffer) -> None:
+        self.__dict__["_buffer"] = __buffer
+        self.setcookieparam = __buffer.setcookieparam
+        self.cipher = __buffer.cipher
+        self.negotiated_protocol = __buffer.negotiated_protocol
+        self.negotiated_tls_version = __buffer.negotiated_tls_version
+
     @property
     def _socket(self) -> _pysocket.socket:
         return cast(_pysocket.socket, self.__dict__["_socket"])
@@ -347,22 +395,6 @@ class TLSWrappedSocket:
                 amt = self._socket.send(in_transit)
                 self._buffer.consume_outgoing(amt)
 
-    def setcookieparam(self, param: bytes) -> None:
-        self._buffer.setcookieparam(param)
-
-    def cipher(self) -> Any:
-        return self._buffer.cipher()
-
-    def negotiated_protocol(self) -> Any:
-        return self._buffer.negotiated_protocol()
-
-    @property
-    def context(self) -> _BaseContext:
-        return self._buffer.context
-
-    def negotiated_tls_version(self) -> Union[TLSVersion, DTLSVersion]:
-        return self._buffer.negotiated_tls_version()
-
     def unwrap(self) -> _pysocket.socket:
         self._buffer.shutdown()
         return self._socket
