@@ -22,8 +22,15 @@ from typing import (
 import pytest
 
 from mbedtls import hashlib
+from mbedtls._tls import (
+    _SUPPORTED_DTLS_VERSION,
+    _SUPPORTED_TLS_VERSION,
+    _dtls_from_version,
+    _dtls_to_version,
+)
 from mbedtls._tls import _DTLSCookie as DTLSCookie  # type: ignore
 from mbedtls._tls import _PSKSToreProxy as PSKStoreProxy  # type: ignore
+from mbedtls._tls import _tls_from_version, _tls_to_version
 from mbedtls.pk import ECC, RSA
 from mbedtls.tls import (
     ClientContext,
@@ -213,53 +220,14 @@ class TestPSKStoreProxy:
         assert len(proxy) == len(psk_store)
 
 
-class TestTLSVersion:
-    @pytest.mark.parametrize("version", TLSVersion)
-    def test_major(self, version: TLSVersion) -> None:
-        assert version.major() == 3
+class TestVersion:
+    @pytest.mark.parametrize("version", _SUPPORTED_TLS_VERSION)
+    def test_tls_enum_to_protocol_version(self, version: TLSVersion) -> None:
+        assert _tls_to_version(_tls_from_version(version)) is version
 
-    def test_minor(self) -> None:
-        # assert TLSVersion.SSLv3.minor() == 0
-        assert TLSVersion.TLSv1.minor() == 1
-        assert TLSVersion.TLSv1_1.minor() == 2
-        assert TLSVersion.TLSv1_2.minor() == 3
-
-    @pytest.mark.parametrize("version", TLSVersion)
-    def test_from_major_minor(self, version: TLSVersion) -> None:
-        assert (
-            TLSVersion.from_major_minor(version.major(), version.minor())
-            is version
-        )
-
-    @pytest.mark.parametrize(
-        "version", [TLSVersion.MINIMUM_SUPPORTED, TLSVersion.MAXIMUM_SUPPORTED]
-    )
-    def test_minmax_supported(self, version: TLSVersion) -> None:
-        assert version in TLSVersion
-
-
-class TestDTLSVersion:
-    @pytest.mark.parametrize("version", DTLSVersion)
-    def test_major(self, version: DTLSVersion) -> None:
-        assert version.major() == 3
-
-    def test_minor(self) -> None:
-        assert DTLSVersion.DTLSv1_0.minor() == 2
-        assert DTLSVersion.DTLSv1_2.minor() == 3
-
-    @pytest.mark.parametrize("version", DTLSVersion)
-    def test_from_major_minor(self, version: DTLSVersion) -> None:
-        assert (
-            DTLSVersion.from_major_minor(version.major(), version.minor())
-            is version
-        )
-
-    @pytest.mark.parametrize(
-        "version",
-        [DTLSVersion.MINIMUM_SUPPORTED, DTLSVersion.MAXIMUM_SUPPORTED],
-    )
-    def test_minmax_supported(self, version: DTLSVersion) -> None:
-        assert version in DTLSVersion
+    @pytest.mark.parametrize("version", _SUPPORTED_DTLS_VERSION)
+    def test_dtls_enum_to_protocol_version(self, version: DTLSVersion) -> None:
+        assert _dtls_to_version(_dtls_from_version(version)) is version
 
 
 class TestTLSRecordHeader:
@@ -268,7 +236,7 @@ class TestTLSRecordHeader:
         assert isinstance(request.param, TLSRecordHeader.RecordType)
         return request.param
 
-    @pytest.fixture(params=TLSVersion)
+    @pytest.fixture(params=_SUPPORTED_TLS_VERSION)
     def version(self, request: Any) -> TLSVersion:
         assert isinstance(request.param, TLSVersion)
         return request.param
@@ -438,17 +406,15 @@ class TestConfiguration:
 
     @pytest.mark.parametrize(
         "inner_protocols",
-        [[], (), [NextProtocol.H2, NextProtocol.H2C], [b"h2", b"h2c", b"ftp"]],
+        [(), (NextProtocol.H2, NextProtocol.H2C), (b"h2", b"h2c", b"ftp")],
     )
     def test_set_inner_protocols(
         self,
         conf: Union[TLSConfiguration, DTLSConfiguration],
-        inner_protocols: Sequence[Union[NextProtocol, bytes]],
+        inner_protocols: Tuple[Union[NextProtocol, bytes]],
     ) -> None:
         conf_ = conf.update(inner_protocols=inner_protocols)
-        assert conf_.inner_protocols == tuple(
-            NextProtocol(_) for _ in inner_protocols
-        )
+        assert conf_.inner_protocols == inner_protocols
 
     @pytest.mark.parametrize("store", [TrustStore.system()])
     def test_trust_store(
@@ -478,15 +444,13 @@ class TestConfiguration:
         conf_ = conf.update(pre_shared_key=psk)
         assert conf_.pre_shared_key == psk
 
-    @pytest.mark.parametrize(
-        "psk_store", [None, {"client": b"the secret key"}]
-    )
+    @pytest.mark.parametrize("psk_store", [{}, {"client": b"the secret key"}])
     def test_psk_store(
         self,
         conf: Union[TLSConfiguration, DTLSConfiguration],
         psk_store: Mapping[str, bytes],
     ) -> None:
-        assert conf.pre_shared_key_store is None
+        assert not conf.pre_shared_key_store
         conf_ = conf.update(pre_shared_key_store=psk_store)
         assert conf_.pre_shared_key_store == psk_store
 
@@ -496,14 +460,18 @@ class TestTLSConfiguration:
     def conf(self) -> TLSConfiguration:
         return TLSConfiguration()
 
-    @pytest.mark.parametrize("version", TLSVersion)
+    def test_default_supported_versions(self, conf: TLSConfiguration) -> None:
+        assert conf.lowest_supported_version is TLSVersion.TLSv1
+        assert conf.highest_supported_version is TLSVersion.MAXIMUM_SUPPORTED
+
+    @pytest.mark.parametrize("version", _SUPPORTED_TLS_VERSION)
     def test_lowest_supported_version(
         self, conf: TLSConfiguration, version: TLSVersion
     ) -> None:
         conf_ = conf.update(lowest_supported_version=version)
         assert conf_.lowest_supported_version is version
 
-    @pytest.mark.parametrize("version", TLSVersion)
+    @pytest.mark.parametrize("version", _SUPPORTED_TLS_VERSION)
     def test_highest_supported_version(
         self, conf: TLSConfiguration, version: TLSVersion
     ) -> None:
@@ -516,14 +484,18 @@ class TestDTLSConfiguration:
     def conf(self) -> DTLSConfiguration:
         return DTLSConfiguration()
 
-    @pytest.mark.parametrize("version", DTLSVersion)
+    def test_default_supported_versions(self, conf: DTLSConfiguration) -> None:
+        assert conf.lowest_supported_version is DTLSVersion.DTLSv1_0
+        assert conf.highest_supported_version is DTLSVersion.MAXIMUM_SUPPORTED
+
+    @pytest.mark.parametrize("version", _SUPPORTED_DTLS_VERSION)
     def test_lowest_supported_version(
         self, conf: DTLSConfiguration, version: DTLSVersion
     ) -> None:
         conf_ = conf.update(lowest_supported_version=version)
         assert conf_.lowest_supported_version is version
 
-    @pytest.mark.parametrize("version", DTLSVersion)
+    @pytest.mark.parametrize("version", _SUPPORTED_DTLS_VERSION)
     def test_highest_supported_version(
         self, conf: DTLSConfiguration, version: DTLSVersion
     ) -> None:
@@ -544,7 +516,7 @@ class TestDTLSConfiguration:
     def test_handshake_timeout_minmax(
         self,
         conf: DTLSConfiguration,
-        hs_min_max: Tuple[Optional[int], Optional[int]],
+        hs_min_max: Tuple[float, float],
     ) -> None:
         hs_min, hs_max = hs_min_max
         assert conf.handshake_timeout_min == 1.0
@@ -556,21 +528,22 @@ class TestDTLSConfiguration:
         assert conf_.handshake_timeout_min == hs_min
         assert conf_.handshake_timeout_max == hs_max
 
-    @pytest.mark.parametrize(
-        "hs_min_max", [(None, None), (1, None), (None, 60)]
-    )
+    @pytest.mark.parametrize("hs_min_max", [(1, 60), (42, 69), (4.2, 6.9)])
     def test_handshake_timeout_default(
         self,
         conf: DTLSConfiguration,
-        hs_min_max: Tuple[Optional[int], Optional[int]],
+        hs_min_max: Tuple[float, float],
     ) -> None:
+        assert conf.handshake_timeout_min == 1.0
+        assert conf.handshake_timeout_max == 60.0
+
         hs_min, hs_max = hs_min_max
         conf_ = conf.update(
             handshake_timeout_min=hs_min,
             handshake_timeout_max=hs_max,
         )
-        assert conf_.handshake_timeout_min == hs_min or 1.0
-        assert conf_.handshake_timeout_max == hs_max or 60.0
+        assert conf_.handshake_timeout_min == hs_min
+        assert conf_.handshake_timeout_max == hs_max
 
 
 class TestContext:
@@ -603,8 +576,7 @@ class TestContext:
         context: Union[ServerContext, ClientContext],
         conf: Union[TLSConfiguration, DTLSConfiguration],
     ) -> None:
-        assert conf
-        assert context.configuration is conf
+        assert context.configuration == conf
 
 
 CLIENT_HELLO = (HandshakeStep.CLIENT_HELLO,)
@@ -861,7 +833,7 @@ class TestDTLSHandshake:
         client = make_client(
             DTLSConfiguration(
                 pre_shared_key=psk,
-                ciphers=["TLS-ECDHE-PSK-WITH-CHACHA20-POLY1305-SHA256"],
+                ciphers=("TLS-ECDHE-PSK-WITH-CHACHA20-POLY1305-SHA256",),
                 lowest_supported_version=DTLSVersion.DTLSv1_2,
                 validate_certificates=False,
             ),
